/* Server functions
 * src/server.c
 * Copyright (C) 2000-2001 DarkSiderz Net.  All rights reserved.
 * Read the file doc/contact.html for contact information related to 
 * this software. */

/*************************************************************************/

#ifndef SERVER_C
#define SERVER_C

/*************************************************************************/

#include "services.h"

/*************************************************************************/

static ServerInfo *serverlists[256];
static void alpha_insert_server FUNCTION((ServerInfo *si));

/*************************************************************************/

/* delserver()
 * Delete server from the server structure. */

int delserver(ServerInfo *si)
{
   int i;

   if (si->next)
      si->next->prev = si->prev;
   if (si->prev)
      si->prev->next = si->next;
   else
      serverlists[tolower(*si->server)] = si->next;
   free(si);
   return 1;
}

/*************************************************************************/

/* findserver()
 * Find a server in the server lists and return its structure
 * information. */

ServerInfo *findserver(char *server)
{
   ServerInfo *si;
   int i;

   for (si = serverlists[tolower(*server)]; si; si = si->next)
      if (stricmp(si->server, server) == 0)
         return si;
   return NULL;
}

/*************************************************************************/

/* add_server()
 * Add a server to the server list. */

ServerInfo *add_server(const char *server)
{
   ServerInfo *si;

   si = scalloc(sizeof(ServerInfo), 1);
   strscpy(si->server, server, SERVERMAX);
   alpha_insert_server(si);
   return si;
}

/*************************************************************************/

/* alpha_insert_server()
 * Insert a server alphabetically into the server structure. */

static void alpha_insert_server(ServerInfo *si)
{
   ServerInfo *ptr, *prev;
   char *server = si->server;

   for (prev = NULL, ptr = serverlists[tolower(*server)]; ptr && stricmp(ptr->server, server) < 0; prev = ptr, ptr = ptr->next)
   ;
   si->prev = prev;
   si->next = ptr;
   if (!prev)
      serverlists[tolower(*server)] = si;
   else
      prev->next = si;
   if (ptr)
      ptr->prev = si;
}

/*************************************************************************/

/* send_map()
 * Send the current network mapping of all the connected servers, showing
 * the number of users, percent of users on this server in relation to the
 * network, the lag from the server to services and the server name. */

void send_map(const char *source, char *client)
{
   ServerInfo *si;
   char *desc;
   int i, done = 0, servers = 0, users = 0, lag = 0, tusers = (lu_users + lu_invisible);
   
   notice(client, source, "Current network server mapping:");
   notice(client, source, "Users Percent Lag Server");


   for (i = 0; i < 256; i++) {
      for (si = serverlists[i]; si; si = si->next) {
	   desc = sstrdup(si->desc);
/*
         while (!done) {
		if (*desc == ']') {
			*desc++;
			*desc++;
			done = 1;
		} else {
			*desc++;
		}
	   }		
*/
         if (si->flags & SI_CONNECT && !(si->flags & SI_JUPE)) {
            if (((float) (si->usrcnt * 100) / (float) tusers) < 10)
               notice(client, source, "%5d   %3.2f%% %3d %s (%s)",
                  si->usrcnt, (float) (si->usrcnt * 100) / (float) tusers, si->ping, si->server, desc);    
            else if (((float) (si->usrcnt * 100) / (float) tusers) == 100)
               notice(client, source, "%5d %3.2f%% %3d %s (%s)",
                  si->usrcnt, (float) (si->usrcnt * 100) / (float) tusers, si->ping, si->server, desc);    
            else
               notice(client, source, "%5d  %3.2f%% %3d %s (%s)",
                  si->usrcnt, (float) (si->usrcnt * 100) / (float) tusers, si->ping, si->server, desc);    
            servers++;
            users += si->usrcnt;
            lag += si->ping;
         }
      }
   }
}

/*************************************************************************/

/* send_pings()
 * Ping all servers which are connected to the network. */

void send_pings(void)
{
   ServerInfo *si;
   int i;

   if (conf.sendpings) {
      for (i = 0; i < 256; i++) {
         for (si = serverlists[i]; si; si = si->next) {
            if (si->flags & SI_CONNECT && !(si->flags & SI_JUPE)) {
               si->ping = 0;
               send_cmd(conf.lserver, "PING %s :%s", conf.lserver, si->server);
            }
         }
      } 
   }
}

/*************************************************************************/

/* free_all_server()
 * Free all the server structures. */

void free_all_server(void)
{
   ServerInfo *si;
   int i;

   for (i = 0; i < 256; i++)
      for (si = serverlists[i]; si; si = si->next)
         delserver(si);
}

/*************************************************************************/

/* os_list_stats_h()
 * List the /stats h command. */

void os_list_stats_h(const char *source)
{
   ServerInfo *si;
   int i;

   for (i = 0; i < 256; i++) {
      for (si = serverlists[i]; si; si = si->next) {
         if (si->hops == 1)
	   send_cmd(NULL, "244 %s :H * * %s 0 -1", source, si->server);
      }
   }
}

/*************************************************************************/

/* is_linked()
 * Are we connected to the hub now? */

void is_linked(void)
{
   ServerInfo *si;
   int i;

   for (i = 0; i < 256; i++) {
      for (si = serverlists[i]; si; si = si->next) {
	   if (si->hops == 1 && hub.tscur == TS_CURRENT && hub.tsmin == TS_MIN) {
		  globops(NULL, "Link with %s[(+)localhost@%s] established: Hub TS (%d->%d) Link.",
			si->server, conf.rserver, TS_CURRENT, hub.tscur);    
	           run.flags &= ~RUN_LINKED;
	   } else {
		  globops(NULL, "Link with %s[(+)localhost@%s] established: Hub Non-TS (%d->%d) Link.",
			si->server, conf.rserver, TS_CURRENT, hub.tscur);    
	           run.flags &= ~RUN_LINKED;
	   }
      }
   }
}

/*************************************************************************/

/* is_synced()
 * Are we synced to the hub now? */

void is_synced(void)
{
   ServerInfo *si;
   int i;

   for (i = 0; i < 256; i++) {
      for (si = serverlists[i]; si; si = si->next) {
	 if (si->hops == 1 && si->ping <= 1) {
          globops(NULL, "DS has successfully synced to the network data, going into normal operation...");
	  }
  	  run.flags &= ~RUN_SYNCING;
      }
   }
}

/*************************************************************************/

#endif

/*************************************************************************/
